//---------------------------------------------------------------------------
//
//   FixVertex
//
//          Copyright(C) 2014, Tark
//
//  I𒸓_Œ
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <algorithm>

#include "FixVertexWindow.h"
#include "FixVertex.h"
#include "MQSetting.h"

static FixVertex plugin;

const float FixVertex::delta = 0.001f;
// Constructor
// RXgN^
FixVertex::FixVertex() : m_Window(nullptr), m_bActivate(false)
{
	setlocale(LC_CTYPE, "");
	for(int i = 0; i < STORENUM; i++) {
		m_isfix[i] = false;
		m_fixmode[i] = 0x7;
	}
	m_fixmode[0] = 0x1; // X0̏ݒX̂݌Œ
}

// Destructor
// fXgN^
FixVertex::~FixVertex()
{
	if(m_Window != NULL) {
		delete m_Window;
		m_Window = NULL;
	}
}

//---------------------------------------------------------------------------
//  GetPlugInID
//---------------------------------------------------------------------------
void FixVertex::GetPlugInID(DWORD *Product, DWORD *ID)
{
	*Product = 0xE3F7A830;
	*ID = 0xBD9A187C;
}

//---------------------------------------------------------------------------
//  GetPlugInName
//---------------------------------------------------------------------------
const char *FixVertex::GetPlugInName(void)
{
	return "FixVertex Ver.1.2   Copyright(C) 2014, Tark.";
}

//---------------------------------------------------------------------------
//  EnumString
//---------------------------------------------------------------------------
const char *FixVertex::EnumString(void)
{
	return "FixVertex";
}

//---------------------------------------------------------------------------
//  Initialize
//---------------------------------------------------------------------------
BOOL FixVertex::Initialize()
{
	int posX = -1, posY = -1;
	{
		MQSetting *setting = OpenSetting();
		setting->Load("PosX", posX, -1);
		setting->Load("PosY", posY, -1);
		CloseSetting(setting);
	}
	if(m_Window == NULL) {
		MQWindowBase mainwnd = MQWindow::GetMainWindow();
		m_Window = new FixVertexWindow(this, mainwnd);
		if(posX >= 0 && posY >= 0) {
			m_Window->SetPosX(posX);
			m_Window->SetPosY(posY);
		}
	}
	Activate(nullptr, m_bActivate);
	return TRUE;
}

//---------------------------------------------------------------------------
//  Exit
//    AvP[V̏I
//---------------------------------------------------------------------------
void FixVertex::Exit()
{
	{
		MQSetting *setting = OpenSetting();
		setting->Save("PosX", m_Window->GetPosX());
		setting->Save("PosY", m_Window->GetPosY());
		CloseSetting(setting);
	}
	if(m_Window != NULL) {
		delete m_Window;
		m_Window = NULL;
	}
}

//---------------------------------------------------------------------------
//  Activate
//---------------------------------------------------------------------------
BOOL FixVertex::Activate(MQDocument doc, BOOL flag)
{
	//	m_doc = doc;
	if(m_Window == NULL) {
		return FALSE;
	}
	m_bActivate = flag ? true : false;
	m_Window->SetVisible(m_bActivate);

	return m_bActivate;
}

//---------------------------------------------------------------------------
//  IsActivated
//---------------------------------------------------------------------------
BOOL FixVertex::IsActivated(MQDocument doc)
{
	if(m_Window == NULL) {
		return FALSE;
	}
	return m_bActivate;
}

//---------------------------------------------------------------------------
//  ExecuteCallback
///---------------------------------------------------------------------------
bool FixVertex::ExecuteCallback(MQDocument doc, void *option)
{
	CallbackInfo *info = (CallbackInfo*)option;
	return ((*this).*info->proc)(doc);
}

// R[obŇĂяo
void FixVertex::Execute(ExecuteCallbackProc proc)
{
	CallbackInfo info;
	info.proc = proc;
	BeginCallback(&info);
}

// vOC̃x[XNXԂ
MQBasePlugin *GetPluginClass()
{
	return &plugin;
}

void FixVertex::OnEndDocument(MQDocument doc_)
{
	clear();
}

void FixVertex::OnDraw(MQDocument doc_, MQScene scene, int width, int height)
{
	if(!m_bActivate) {
		return;
	}
	cmq::CMQDocument doc(doc_);
	//cmq::CMQObject obj(doc.getCurrentObject());
	for(int i = 0; i < STORENUM; i++) {
		if(m_isfix[i]) {
			for(auto searchit : m_fixedvertex[i]) {
				UINT ouid = searchit.first;
				cmq::CMQObject obj = doc.getObjectFromUniqueID(ouid);
				if(!obj.getVisible() || obj.getLocking()) {
					continue;
				}
				auto &vmap = searchit.second;
				for(auto it = vmap.begin(); it != vmap.end(); ++it) {
					cmq::vindex_t vindex = obj.getVertexIndexFromUniqueID(it->first);
					if(vindex != -1) {
						MQPoint &p = it->second;
						if(m_fixmode[i] != 0x7) {
							MQPoint p1 = obj.getVertex(vindex);
							if(!(m_fixmode[i] & 0x1)) { p.x = p1.x; }
							if(!(m_fixmode[i] & 0x2)) { p.y = p1.y; }
							if(!(m_fixmode[i] & 0x4)) { p.z = p1.z; }
						}
						obj.setVertex(vindex, p);
					}
				}
			}
		}
	}
}

bool FixVertex::fix(MQDocument doc_, int index, bool isfix)
{
	if(index == 0) {
		// X0̒_L
		m_fixedvertex[0].clear();
		cmq::CMQDocument doc(doc_);
		int ocount = doc.getObjectCount();
		for(int oindex = 0; oindex < ocount; oindex++) {
			cmq::CMQObject obj = doc.getObject(oindex);
			if(!obj.isLive()) { continue; }
			if(!obj.getVisible()) { continue; }
			// IuWFNgSċL
			UINT ouid = obj.getUniqueID();
			std::unordered_map<UINT, MQPoint> vmap0;
			auto insertit = m_fixedvertex[0].insert(std::make_pair(ouid, vmap0));
			auto &vmap = insertit.first->second;
			cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
			for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
				MQPoint p = obj.getVertex(vit.index());
				if(abs(p.x) < delta) {
					vmap.insert(std::make_pair(obj.getVertexUniqueID(vit.index()), obj.getVertex(vit.index())));
				}
			}
		}
	}
	isfix = isfix && !m_fixedvertex[index].empty();
	return m_isfix[index] = isfix;
}

bool FixVertex::setselected(MQDocument doc_, int index, bool isfix, bool isappend)
{
	if(!m_bActivate) {
		return false;
	}
	cmq::CMQDocument doc(doc_);
	cmq::CMQObject obj(doc.getCurrentObject());
	UINT ouid = obj.getUniqueID();
	auto findit = m_fixedvertex[index].find(ouid);
	if(findit == m_fixedvertex[index].end()) {
		std::unordered_map<UINT, MQPoint> vmap0;
		auto insertit = m_fixedvertex[index].insert(std::make_pair(ouid, vmap0));
		findit = insertit.first;
	}
	auto &vmap = findit->second;
	if(isfix) {
		// I𒸓_W
		if(!isappend) {
			vmap.clear();
		}
		cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
		for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
			if(vit->isSelect()) {
				vmap.insert(std::make_pair(obj.getVertexUniqueID(vit.index()), obj.getVertex(vit.index())));
			}
		}
	}
	m_isfix[index] = !m_fixedvertex[index].empty();
	return m_isfix[index];
}

void FixVertex::clear(MQDocument doc, int index)
{
	m_fixedvertex[index].clear();
	m_isfix[index] = false;
}

void FixVertex::clear()
{
	for(int i = 0; i < STORENUM; i++) {
		m_fixedvertex[i].clear();
		m_isfix[i] = false;
	}
	m_Window->clear();
}